#ifndef _UNSTUFF_H
#define _UNSTUFF_H 1

#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>

// Copies from src to dst, removing all instances of 0xFF 0xF0.
//
// Returns:
//   Number of bytes written to dst if successful.
//   -1 if a marker was encountered (0xFF followed by something else than 0xF0).
//   -1 if the last byte of "src" was 0xFF. 

typedef int (unstuff_func_t)(uint8_t*, const uint8_t*, size_t);

// Slow reference function.
int unstuff_reference(uint8_t* dst, const uint8_t* src, size_t len);

// Faster version (assuming few stuff bytes), using C standard library
// functions (that are hopefully more efficient, assuming few stuff bytes)
// to find the markers.
// Slightly over twice as fast as unstuff_reference().
int unstuff_fast(uint8_t* dst, const uint8_t* src, size_t len);

// SSE4.1 version working on 16 bytes at a time.
// 5-10% faster than unstuff_fast().
int unstuff_sse41(uint8_t* dst, const uint8_t* src, size_t len);

#endif /* !defined(_UNSTUFF_H) */
